class AuthManager::Validation

  def self.validate_param(hash)
    result = validate_format_of(hash)
    result = validate_option(hash[:option]) if result[:code] == 200
    return result if (result[:code] == 400)
    return result = {code: 400, message: 'Bad object and option association'} unless validate_param_association(hash)
    result = user_as_param(hash[:object]) if result[:code] && hash[:option][:user]
    result = app_as_param(hash[:object]) if result[:code] && hash[:option][:app]
    result
  end

  private ################################

  def self.validate_param_association(hash)
    good_association = true
    good_association = false if (user_as_param(hash[:object])[:code] == 200) && (hash[:option][:app])
    good_association = false if (app_as_param(hash[:object])[:code] == 200) && (hash[:option][:user])
    good_association
  end

  def self.user_as_param(object)
    answer = {code: 400, message: 'Object not well formed'}
    answer = {code: 400, message: 'Object is not a hash'} unless
              object.is_a?(Hash)
    answer = {code: 200, message: 'Object is valid'} if (object_has2keys(object) || object_has3keys(object)) && object_contains_string(object)
    answer
  end

  def self.app_as_param(object)
    answer = {code: 400, message: 'Object not well formed'}
    answer = {code: 400, message: 'Object is not a hash'} unless
              object.is_a?(Hash)
    answer = {code: 200, message: 'Object is valid'} if (object_has1key(object) || object_has2keys(object)) && object_is_string(object)
    answer
  end

  def self.object_has1key(object)
    object.include?(:api_key) && (object.length == 1)
  end

  def self.object_is_string(object)
    object[:api_key].is_a?(String) && !object[:api_key].empty?
  end

  def self.object_has2keys(object)
    ((object.include?(:login) && (object.include?(:password)))|| (object.include?(:api_key) && object.include?(:deleted))) && (object.length == 2)
  end

  def self.object_has3keys(object)
    (object.include?(:login) && object.include?(:password) && object.include?(:deleted) && (object.length == 3))
  end

  def self.object_contains_string(object)
    object[:login].is_a?(String) && object[:password].is_a?(String) && !object[:login].empty? && !object[:password].empty?
  end

  def self.validate_option(option)
    answer = {code: 400, message: 'Option not well formed'}
    answer = {code: 400, message: 'Option is not a hash'} unless
              option.is_a?(Hash)
    answer = {code: 200, message: 'Option is valid'} if option_has2keys(option) && option_keys_are_bools(option) && only_one_option(option)
    answer
  end

  def self.option_has2keys(option)
    (option && option.include?(:user) && option.include?(:app) && (option.length == 2))
  end

  def self.option_keys_are_bools(option)
    option && (option[:user].is_a?(TrueClass) || option[:user].is_a?(FalseClass)) && (option[:app].is_a?(TrueClass) || option[:app].is_a?(FalseClass))
  end

  def self.only_one_option(option)
    (option && option[:user] && ! option[:app]) || (option && !option[:user] && option[:app])
  end

  def self.validate_format_of(hash)
    answer = {code: 400, message: 'Param not well formed!'}
    answer = {code: 400, message: 'Param not a hash!'} unless hash.is_a?(Hash)
    (answer = {code: 200, message: 'Param is valid.'}) if (hash.is_a?(Hash) && hash.include?(:object) && hash.include?(:option))
    answer
  end
end
